# -*- coding: utf-8 -*-

from view.ASubWindow import ASubWindow
from model.network.Node import Node
from model.network.Edge import Edge
from model.network.Graph import Graph

from PyQt5.QtCore import (
    Qt, QRect, QVariant, QAbstractTableModel, pyqtSlot, pyqtSignal,
)

from PyQt5.QtWidgets import (
    QTableView, QItemDelegate, QComboBox, QLineEdit,
)

class LineEditDelegate(QItemDelegate):
    on_focus_out = pyqtSignal(object)

    def __init__(self, *args, **kwargs):
        super(LineEditDelegate, self).__init__(*args, **kwargs)
        self.line_edit = None

    def createEditor(self, parent, option, index):
        self.line_edit = QLineEdit(parent=parent)
        self.line_edit.destroyed.connect(lambda: self._line_edit_left(index=index))
        return self.line_edit

    def _line_edit_left(self, index):
        self.on_focus_out.emit(index)
        return

class ComboBoxDelegate(QItemDelegate):
    def __init__(self, graph=None, parent=None):
        super(ComboBoxDelegate, self).__init__(parent)
        self.graph = graph

    def createEditor(self, parent, option, index):
        self.editor = QComboBox(parent)
        self.editor.addItems(self.graph.nodes_names())
        return self.editor

    def setEditorData(self, editor, index):
        value = index.data(Qt.DisplayRole)
        self.editor.currentTextChanged.connect(self.currentItemChanged)

    def setModelData(self, editor, model, index):
        text = str(editor.currentText())
        model.setData(index, text)
        editor.close()
        editor.deleteLater()

    def updateEditorGeometry(self, editor, option, index):
        r = QRect(option.rect)
        if self.editor.windowFlags() & Qt.Popup and editor.parent() is not None:
            r.setTopLeft(self.editor.parent().mapToGlobal(r.topLeft()))
        editor.setGeometry(r)

    @pyqtSlot()
    def currentItemChanged(self):
        self.commitData.emit(self.sender())


class TableModel(QAbstractTableModel):
    def __init__(self, headers=[], rows=[], graph=None):
        super(QAbstractTableModel, self).__init__()
        self.rows = rows
        self.headers = headers
        self.graph = graph

    def flags(self, index):
        return Qt.ItemIsEditable | Qt.ItemIsEnabled | Qt.ItemIsSelectable

    def rowCount(self, parent):
        return len(self.rows)

    def columnCount(self, parent):
        return len(self.headers)

    def data(self, index, role):
        if role != Qt.ItemDataRole.DisplayRole:
            return QVariant()

        return self.rows[index.row()][self.headers[index.column()]]

    def headerData(self, section, orientation, role):
        if role == Qt.ItemDataRole.DisplayRole and orientation == Qt.Orientation.Horizontal:
            return self.headers[section].capitalize()

        if role == Qt.ItemDataRole.DisplayRole and orientation == Qt.Orientation.Vertical:
            return section

        return QVariant()

    @pyqtSlot()
    def setData(self, index, value, role=Qt.EditRole):
        if index.isValid():
            if role == Qt.EditRole:
                if (self.headers[index.column()] == "node1" or
                    self.headers[index.column()] == "node2"):
                    node = self.graph.node(value)
                    self.rows[index.row()][self.headers[index.column()]] = node
                else:
                    self.rows[index.row()][self.headers[index.column()]] = value

                self.dataChanged.emit(index, index, [Qt.DisplayRole])
                self.layoutChanged.emit()
                return True

            self.dataChanged.emit(index, index)
        else:
            return False

    def update(self):
        print("update")
        self.layoutChanged.emit()


class NetworkWindow(ASubWindow):
    def __init__(self, title="Network", parent=None):
        super(NetworkWindow, self).__init__(name=title, ui="Network", parent=parent)
        self.ui.setWindowTitle(title)

        self.graph = Graph()

        # Nodes table

        self.nodes_model = TableModel(
            headers = ["name", "id"],
            rows = self.graph.nodes(),
            graph = self.graph,
        )
        self.delegate_line = LineEditDelegate(parent=self)
        table = self.find(QTableView, "tableView_nodes")
        table.setModel(self.nodes_model)
        table.setItemDelegate(self.delegate_line)
        #table.resizeColumnsToContents()

        # Edges table

        self.reachs_model = TableModel(
            headers = ["name", "node1", "node2"],
            rows = self.graph.edges(),
            graph=self.graph,
        )
        self.delegate_combobox = ComboBoxDelegate(
            graph=self.graph,
            parent=self,
        )
        table = self.find(QTableView, "tableView_reachs")
        table.setModel(self.reachs_model)
        table.setItemDelegateForColumn(1, self.delegate_combobox)
        table.setItemDelegateForColumn(2, self.delegate_combobox)
        #table.resizeColumnsToContents()

        # Connection the two table
        self.nodes_model.dataChanged.connect(self.reachs_model.update)
        self.reachs_model.dataChanged.connect(self.nodes_model.update)
